//This program code is released under GNU General Public License (GPL)
//(http://www.gnu.org/copyleft/gpl.html)
//(C) Wolfgang Gossel (2010)
// Muehlweg 49 
// D-06114 Halle
//or: Martin Luther University
//    Dept. Hydrogeology and Environmental Geology
//    V.-Seckendorff Platz 3
//    D-06120 Halle
//email: wolfgang.gossel@geo.uni-halle.de
//homepage: http://www.mo2geo.org

#include <cstdlib>
#include "get_grid.h"
#include "get_projectFile.h"
#include <math.h>
#include <string>
#include <gl/Gl.h>
#include <gl/Glu.h>
#include <gl/glut.h>


using namespace std;

void showBorehole(void);
void handleResize(int w, int h);
void myInit(void);
void RenderString(float x, float y, void *font, char* string);

double *bounds;
long num_faces;
long numLayers;
long points_per_layer;
long points_per_slice;
long numberLayers;
double *pointX;
double *pointY;
double *pointZ;
long *vertexID1;
long *vertexID2;
long *vertexID3;
float *red;
float *green;
float *blue;
float *alpha;
double prjParams[3];
double *zValue;
string *slName;

int main(int argc, char *argv[])
{
//long num_faces;
//long number_of_points;
long cols, rows, counter1, a;
double x;

getProjectFile projectData;
projectData.setFile(argv[1]);
projectData.getLinesProjectFile();
projectData.analyzeProjectFileContent();
numberLayers=projectData.getNumberSlices();
numLayers=numberLayers;

char *sliceFile[numLayers+1];

red=new float[numLayers+1];
green=new float[numLayers+1];
blue=new float[numLayers+1];
alpha=new float[numLayers+1];
slName=new string[numLayers+1];

for(int j=0;j<numLayers;j++)
{
    sliceFile[j]=projectData.getPathFileSlice(j);
    slName[j]=projectData.getNameSlice(j);
    red[j]=(float(projectData.getRed(j)))/256;
    green[j]=(float(projectData.getGreen(j)))/256;
    blue[j]=(float(projectData.getBlue(j)))/256;
    alpha[j]=1;
}

projectData.getProjectionParams(prjParams);

getGRID sliceGRID;
sliceGRID.setFile(sliceFile[0]);
sliceGRID.getPointsFromFile();
sliceGRID.analyze_file_content();
points_per_slice=sliceGRID.getNumberPoints();


double *pointX_sl=new double[points_per_slice];
double *pointY_sl=new double[points_per_slice];
double *pointZ_sl=new double[points_per_slice];

double *pointX_slice=new double[points_per_slice*(numLayers+1)];
double *pointY_slice=new double[points_per_slice*(numLayers+1)];
double *pointZ_slice=new double[points_per_slice*(numLayers+1)];

double *bounds=new double[6];

sliceGRID.getBoundaries(bounds);
cols=sliceGRID.getNumberCols();
rows=sliceGRID.getNumberRows();

for (long counterSlices=0; counterSlices<numLayers;counterSlices++)
{
    
    sliceGRID.setFile(sliceFile[counterSlices]);
    sliceGRID.getPointsFromFile();
    sliceGRID.analyze_file_content();
    sliceGRID.getPoints(pointX_sl, pointY_sl, pointZ_sl);
    
    for(long i=0; i<points_per_slice;i++)
    {
        pointX_slice[i+counterSlices*points_per_slice]=pointX_sl[i];
        pointY_slice[i+counterSlices*points_per_slice]=pointY_sl[i];
        pointZ_slice[i+counterSlices*points_per_slice]=pointZ_sl[i];
    }
}

//calculate index of virtual borehole

double xDiff=0, colWidth=0, yDiff=0, rowWidth=0;
long colNumX=0, rowNumY=0, index;
xDiff=(prjParams[0]-bounds[0]);
colWidth=(bounds[1]-bounds[0])/cols;
colNumX=(long)(xDiff/colWidth);
yDiff=(prjParams[1]-bounds[2]);
rowWidth=(bounds[3]-bounds[2])/rows;
rowNumY=(long)(yDiff/rowWidth);
index=rowNumY*cols+colNumX;

zValue=new double[numLayers];

for (long i=0;i<numLayers;i++)
{
    zValue[i]=pointZ_slice[index+i*points_per_slice];
    
}

glutInit(&argc, argv);
glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
glutInitWindowSize(200, 800);
glutInitWindowPosition(100,150);
glutCreateWindow("Virtual Borehole");
glutDisplayFunc(showBorehole);
myInit();
glViewport(0,0,400,800);
//glutReshapeFunc(handleResize);
glutMainLoop();

return(0);

}


void showBorehole(void)
{   

    float refl[4]={0.5,0.3,0.8,0.4};
    float z=0.9;
    
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    glBegin(GL_QUADS);
    glEnable(GL_COLOR_MATERIAL);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, refl);

    for (int l=0; l<numberLayers; l++)
    {
        glColor4f(red[l], green[l], blue[l], 1);
        glVertex2f(-0.3, z);
        glVertex2f(0.0, z);
        z=z+(zValue[l+1]-zValue[l])/prjParams[2];
        glVertex2f(0.0, z);
        glVertex2f(-0.3, z);
        //glColor3f(0.0, 1.0, 0.0);
        
    }
    
    glEnd();

    char heightSlice[7];
    int isConv;
    z=0.9;
    double thickness=0;
    for (int l=0; l<numberLayers; l++)
    {
        sprintf(heightSlice, "%4.2f", zValue[l]);
        RenderString(-0.9, z, GLUT_BITMAP_9_BY_15, heightSlice);
        z=z+(zValue[l+1]-zValue[l])/prjParams[2];
    }
    
    z=0.9+(0.5*((zValue[1]-zValue[0])/prjParams[2]));
    

    char sliceNameA[50];
    
    for (int l=0; l<numberLayers-1; l++)
    {
        thickness=zValue[l]-zValue[l+1];
        char *sliceNme = new char[slName[l].size()+1];
        strcpy(sliceNme, slName[l].c_str());
        sliceNme[slName[l].size()+1]='\0';
        sprintf(sliceNameA, "%4.2f", thickness); 
        strcat(sliceNameA,", ");
        strcat(sliceNameA, sliceNme);      
        RenderString(0.1, z, GLUT_BITMAP_9_BY_15, sliceNameA);
        z=z+0.5*(zValue[l+1]-zValue[l])/prjParams[2]+0.5*(zValue[l+2]-zValue[l+1])/prjParams[2];
    }
    glutSwapBuffers();

}

//void handleResize(int width, int height) 
//{
//	glViewport(0, 0, width, height);
//	glMatrixMode(GL_PROJECTION);
//	glLoadIdentity();
//	gluPerspective(30.0f,width/height,0.1f,100.0f);
//	glMatrixMode(GL_MODELVIEW);
//	glLoadIdentity();
//	
//	glClearDepth(1.0);
//	glEnable(GL_DEPTH_TEST);
//	glDepthFunc(GL_LEQUAL);
//	
//	glEnable(GL_CULL_FACE);
//	glCullFace(GL_BACK);
//	glFrontFace(GL_CW);	
//
//	glShadeModel(GL_SMOOTH);
//	GLfloat AmbientLight[] = {0.3, 0.3, 0.3, 1.0};
//	GLfloat DiffuseLight[] = {0.5, 0.5, 0.5, 1.0};
//	GLfloat LightPosition[] = {0.0, 0.0, 0.0, 1.0};
//	GLfloat SpecularLight[] = {1.0, 1.0, 1.0, 1.0};
//	GLfloat SpecularReflection[] = {0.4, 0.4, 0.4, 1.0};
//	
//	glEnable(GL_LIGHTING);
//	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, AmbientLight);
//	glEnable(GL_COLOR_MATERIAL);
//	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
//	
//	glEnable(GL_LIGHTING);
//	glLightfv(GL_LIGHT0,GL_AMBIENT, AmbientLight);
//	glLightfv(GL_LIGHT0,GL_DIFFUSE, DiffuseLight);
//	glLightfv(GL_LIGHT0,GL_SPECULAR, SpecularLight);
//	glEnable(GL_LIGHT0);
//	glLightfv(GL_LIGHT0,GL_POSITION,LightPosition);
//	
//}
//
//
void myInit(void)
{
    glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
}


void RenderString(float x, float y, void *font, char* stringLayer)
{  
  char *c;

  glColor3f(1.0, 1.0, 1.0); 
  glRasterPos2f(x, y);
  
  for(char* c=stringLayer; *c!='\0';c++)
  {
        glutBitmapCharacter(font, *c);
    }
}


